from argparse import FileType, ArgumentError
import json
import logging
from os.path import basename
from pprint import pprint
import re
import tarfile
from warnings import warn

from binstar_client.pprintb import package_list, user_list
from binstar_client.utils import get_server_api, parse_specs
from binstar_client.utils.spec import group_spec



def main(args):

    aserver_api = get_server_api(args.token, args.site, args.log_level)
    spec = args.spec
    action = args.action
    verbose = args.log_level == logging.DEBUG

    if action == 'add':
        aserver_api.add_group(spec.org, spec.group_name, args.perms)
        print('Created the group %s' % (spec,))
    elif action == 'show':
        if spec._group_name:
            result = aserver_api.group(spec.org, spec.group_name)
            pprint(result)
        else:
            result = aserver_api.groups(spec.org)
            pprint(result)

    elif action == 'members':
        result = aserver_api.group_members(spec.org, spec.group_name)
        user_list(result, verbose)
    elif action == 'add_member':
        aserver_api.add_group_member(spec.org, spec.group_name, spec.member)
        print('Added the user "{spec.member}" to the group "{spec.org}/{spec.group_name}"'.format(spec=spec))
    elif action == 'remove_member':
        aserver_api.remove_group_member(spec.org, spec.group_name, spec.member)
        print('Removed the user "{spec.member}" from the group "{spec.org}/{spec.group_name}"'.format(spec=spec))
    elif action == 'packages':
        result = aserver_api.group_packages(spec.org, spec.group_name)
        package_list(result, verbose)
    elif action == 'add_package':
        aserver_api.add_group_package(spec.org, spec.group_name, spec.member)
        print('Added the package "{spec.member}" to the group "{spec.org}/{spec.group_name}"'.format(spec=spec))
    elif action == 'remove_package':
        aserver_api.remove_group_package(spec.org, spec.group_name, spec.member)
        print('Removed the package "{spec.member}" from the group "{spec.org}/{spec.group_name}"'.format(spec=spec))
    else:
        raise NotImplementedError(args.action)

def add_parser(subparsers):

    parser = subparsers.add_parser('groups',
                                    help='Manage Groups',
                                    description=__doc__)

    parser.add_argument('action',
                        choices=['add', 'show', 'members', 'add_member',
                                 'remove_member', 'packages', 'add_package',
                                 'remove_package'],
                        help='The group management command to execute')
    parser.add_argument('spec', type=group_spec,
                        help=group_spec.__doc__)
    parser.add_argument('--perms', choices=['read', 'write', 'admin'], default='read',
                        help='The permission the group should provide')

    parser.set_defaults(main=main)



import unittest
from binstar_client.utils.notebook import inflection


class InflectorsTestCase(unittest.TestCase):
    def test_one_directory(self):
        self.assertEqual(inflection.parameterize(u"Donald E. Knuth"), 'donald-e-knuth')


if __name__ == '__main__':
    unittest.main()



import unittest
from binstar_client.tests.fixture import CLITestCase
from binstar_client.tests.urlmock import urlpatch
from binstar_client.scripts.cli import main
from binstar_client import errors


class Test(CLITestCase):
    @urlpatch
    def test_show(self, urls):
        urls.register(
            method='GET',
            path='/groups/org',
            content='{"groups": [{"name":"grp", "permission": "read"}]}',
        )

        main(['--show-traceback', 'groups', 'show', 'org'], False)

        urls.assertAllCalled()

    @urlpatch
    def test_show_group(self, urls):
        urls.register(
            method='GET',
            path='/group/org/owners',
            content='{"name": "owners", "permission": "read", "members_count": 1, "repos_count": 1}',
        )

        main(['--show-traceback', 'groups', 'show', 'org/owners'], False)

        urls.assertAllCalled()

    @urlpatch
    def test_create(self, urls):
        urls.register(
            method='POST',
            path='/group/org/new_grp',
            status=204,
        )

        main(['--show-traceback', 'groups', 'add', 'org/new_grp'], False)

        urls.assertAllCalled()

    @urlpatch
    def test_create_missing_group(self, urls):
        with self.assertRaisesRegexp(errors.UserError, 'Group name not given'):
            main(['--show-traceback', 'groups', 'add', 'org'], False)

    @urlpatch
    def test_add_member(self, urls):
        urls.register(
            method='PUT',
            path='/group/org/grp/members/new_member',
            status=204,
        )

        main(['--show-traceback', 'groups', 'add_member', 'org/grp/new_member'], False)

        urls.assertAllCalled()

    @urlpatch
    def test_add_member_missing_member(self, urls):
        with self.assertRaisesRegexp(errors.UserError, 'Member name not given'):
            main(['--show-traceback', 'groups', 'add_member', 'org/grp'], False)

    @urlpatch
    def test_remove_member(self, urls):
        urls.register(
            method='DELETE',
            path='/group/org/grp/members/new_member',
            status=204,
        )

        main(['--show-traceback', 'groups', 'remove_member', 'org/grp/new_member'], False)

        urls.assertAllCalled()

    @urlpatch
    def test_packages(self, urls):
        urls.register(
            method='GET',
            path='/group/org/grp/packages',
            content='[{"name": "pkg", "full_name": "org/pkg", "summary": "An org pkg"}]'
        )

        main(['--show-traceback', 'groups', 'packages', 'org/grp'], False)

        urls.assertAllCalled()

    @urlpatch
    def test_add_package(self, urls):
        urls.register(
            method='PUT',
            path='/group/org/grp/packages/pkg',
            status=204,
        )

        main(['--show-traceback', 'groups', 'add_package', 'org/grp/pkg'], False)

        urls.assertAllCalled()

    @urlpatch
    def test_remove_package(self, urls):
        urls.register(
            method='DELETE',
            path='/group/org/grp/packages/pkg',
            status=204,
        )

        main(['--show-traceback', 'groups', 'remove_package', 'org/grp/pkg'], False)

        urls.assertAllCalled()

if __name__ == "__main__":
    unittest.main()



from __future__ import print_function, unicode_literals

# Standard libary imports
from os import path
import unittest

# Local imports
from binstar_client.inspect_package import conda
from binstar_client.utils.notebook.data_uri import data_uri_from


def data_path(filename):
    return path.join(path.dirname(__file__), 'data', filename)


expected_package_data = {
    'name': 'conda_gc_test',
    'summary': 'This is a simple meta-package',
    'license': None,
    'description': '',
    'dev_url': None,
    'doc_url': None,
    'home': None,
    'license_url': None,
    'source_git_url': None,
    'license_family': None,
}

expected_version_data_121 = {
    'home': None,
    'description': '',
    'summary': 'This is a simple meta-package',
    'dev_url': None,
    'doc_url': None,
    'license': None,
    'license_url': None,
    'license_family': None,
    'source_git_url': None,
    'source_git_tag': None,
    'description': '',
    'home_page': None,
    'icon': None,  # The icon if found on the conda folder is uploaded here.
    'version': '1.2.1',
}

expected_version_data_221 = {
    'home': None,
    'description': '',
    'summary': 'This is a simple meta-package',
    'dev_url': None,
    'doc_url': None,
    'license': None,
    'license_url': None,
    'license_family': None,
    'source_git_url': None,
    'source_git_tag': None,
    'description': '',
    'home_page': None,
    'icon': None,  # The icon if found on the conda folder is uploaded here.
    'version': '2.2.1',
}

expected_file_data_121 = {
    'attrs': {
        'arch': 'x86_64',
        'build': 'py27_3',
        'build_number': 3,
        'depends': ['foo ==3*', 'python ==2.7.8'],
        'license': None,
        'machine': 'x86_64',
        'operatingsystem': 'darwin',
        'platform': 'osx',
        'subdir': 'osx-64',
        'target-triplet': 'x86_64-any-darwin',
        'has_prefix': False,
        },
    'basename': 'osx-64/conda_gc_test-1.2.1-py27_3.tar.bz2',
    'dependencies': {
        'depends': [{'name': 'foo', 'specs': [['==', '3']]},
                    {'name': 'python', 'specs': [['==', '2.7.8']]}],
        },
    }

expected_file_data_221 = {
    'attrs': {
        'arch': 'x86_64',
        'build': 'py27_3',
        'build_number': 3,
        'depends': ['foo ==3*', 'python ==2.7.8'],
        #'license': None,
        'machine': 'x86_64',
        'operatingsystem': 'linux',
        'platform': 'linux',
        'subdir': 'linux-64',
        'target-triplet': 'x86_64-any-linux',
        'has_prefix': False,
        },
    'basename': 'linux-64/conda_gc_test-2.2.1-py27_3.tar.bz2',
    'dependencies': {
        'depends': [{'name': 'foo', 'specs': [['==', '3']]},
                    {'name': 'python', 'specs': [['==', '2.7.8']]}],
        },
    }

# Test package application data
# -----------------------------------------------------------------------------
ICON_B64 = data_uri_from(data_path('43c9b994a4d96f779dad87219d645c9f.png'))
app_expected_package_data = {
    'name': u'test-app-package-icon',
    'summary': u'',
    'license': 'LICENSE',
    'license_url': 'http://license.url',
    'license_family': None,
    'description': 'test description',
    'dev_url': 'https://dev.url',
    'doc_url': 'https://doc.url',
    'home': 'http://home.page',
    'source_git_url': 'http://git.url',
}

app_expected_version_data = {
    'home': 'http://home.page',
    'description': 'test description',
    'summary': u'',
    'dev_url': 'https://dev.url',
    'doc_url': 'https://doc.url',
    'source_git_url': 'http://git.url',
    'license': 'LICENSE',
    'license_url': 'http://license.url',
    'license_family': None,
    'source_git_tag': 0.1,
    'description': 'test description',
    'home_page': 'http://home.page',
    'icon': ICON_B64,
    'version': u'0.1',
}


class Test(unittest.TestCase):

    def test_conda_old(self):
        filename = data_path('conda_gc_test-1.2.1-py27_3.tar.bz2')
        with open(filename, 'rb') as fd:
            package_data, version_data, file_data = conda.inspect_conda_package(filename, fd)

        self.assertEqual(expected_package_data, package_data)
        self.assertEqual(expected_version_data_121, version_data)
        self.assertEqual(expected_file_data_121, file_data)

    def test_conda(self):
        filename = data_path('conda_gc_test-2.2.1-py27_3.tar.bz2')
        with open(filename, 'rb') as fd:
            package_data, version_data, file_data = conda.inspect_conda_package(filename, fd)

        self.assertEqual(expected_package_data, package_data)
        self.assertEqual(expected_version_data_221, version_data)
        self.assertEqual(expected_file_data_221, file_data)

    def test_conda_app_image(self):
        filename = data_path('test-app-package-icon-0.1-0.tar.bz2')
        with open(filename, 'rb') as fd:
            package_data, version_data, file_data = conda.inspect_conda_package(filename, fd)

        self.assertEqual(app_expected_package_data, package_data)
        self.assertEqual(app_expected_version_data.pop('icon'), version_data.pop('icon'))
        self.assertEqual(app_expected_version_data, version_data)


if __name__ == "__main__":
    # import sys;sys.argv = ['', 'Test.testName']
    unittest.main()



'''
Created on May 7, 2013

@author: sean
'''
from __future__ import print_function
from pprint import pprint

def package_list(lst, verbose=True):
    if verbose:
        pprint(lst)
    else:
        for pkg in lst:
            print('%-25s %s' % (pkg['full_name'], pkg['summary']))

def user_list(lst, verbose=True):
    if verbose:
        pprint(lst)
    else:
        for user in lst:
            print('%-25s %s' % (user['login'], user['name']))




from __future__ import unicode_literals
from binstar_client.scripts.cli import main
from binstar_client.tests.fixture import CLITestCase
from binstar_client.tests.urlmock import urlpatch
from binstar_client.errors import BinstarError
import unittest
from mock import patch


class Test(CLITestCase):
    @urlpatch
    def test_remove_token_from_org(self, urls):
        remove_token = urls.register(
            method='DELETE',
            path='/authentications/org/orgname/name/tokenname',
            content='{"token": "a-token"}',
            status=201
        )
        main(['--show-traceback', 'auth', '--remove', 'tokenname', '-o', 'orgname'], False)
        self.assertIn('Removed token tokenname', self.stream.getvalue())

        remove_token.assertCalled()

    @urlpatch
    def test_remove_token(self, urls):
        remove_token = urls.register(
            method='DELETE',
            path='/authentications/name/tokenname',
            content='{"token": "a-token"}',
            status=201
        )
        main(['--show-traceback', 'auth', '--remove', 'tokenname'], False)
        self.assertIn('Removed token tokenname', self.stream.getvalue())

        remove_token.assertCalled()


    @urlpatch
    def test_remove_token_forbidden(self, urls):
        remove_token = urls.register(
            method='DELETE',
            path='/authentications/org/wrong_org/name/tokenname',
            content='{"token": "a-token"}',
            status=403
        )
        with self.assertRaises(BinstarError):
            main(['--show-traceback', 'auth', '--remove', 'tokenname', '-o', 'wrong_org'], False)
        self.assertEqual('', self.stream.getvalue())

        remove_token.assertCalled()


if __name__ == '__main__':
    unittest.main()
